import flask

from ...usage import usage
from .main import MainController


def get_editor_bp(controller: MainController):
    bp = flask.Blueprint("transitions", __name__)

    @bp.put("/<string:id>")
    @usage
    def _update_transition(id: str):
        data = flask.request.json
        if not data:
            flask.abort(400)

        transition = controller.update_transition(id, data)
        return transition if transition else None

    return bp



from ..widget_base import Output


class LatexOutput(Output):
    type = "latex-output"

    def __init__(self, text: str, **kwargs):
        self.set_props(dict(text=text, **kwargs))

    def set_props(self, props):
        self.text = props.get("text", "")
        self.full_width = props.get("full_width", False)

    def render(self, ctx: dict):
        return {"type": self.type, "text": self.text, "fullWidth": self.full_width}



from .IframeOutput import IframeOutput
from .NpsInput import NpsInput
from .PandasRowSelectionInput import PandasRowSelectionInput
from .ChecklistInput import ChecklistInput
from .CurrencyInput import CurrencyInput
from .DropdownInput import DropdownInput
from .FileInput import FileInput
from .PhoneInput import PhoneInput
from .CnpjInput import CnpjInput
from .PandasOutput import PandasOutput
from .CheckboxInput import CheckboxInput
from .TextareaInput import TextareaInput
from .ImageInput import ImageInput
from .TimeInput import TimeInput
from .ToggleInput import ToggleInput
from .TagInput import TagInput
from .LatexOutput import LatexOutput
from .MarkdownOutput import MarkdownOutput
from .PasswordInput import PasswordInput
from .NumberInput import NumberInput
from .TextOutput import TextOutput
from .CameraInput import CameraInput
from .CodeInput import CodeInput
from .NumberSliderInput import NumberSliderInput
from .EmailInput import EmailInput
from .PlotlyOutput import PlotlyOutput
from .RatingInput import RatingInput
from .CpfInput import CpfInput
from .TextInput import TextInput
from .ListInput import ListInput
from .LinkOutput import LinkOutput
from .ImageOutput import ImageOutput
from .ProgressOutput import ProgressOutput
from .DateInput import DateInput
from .HtmlOutput import HtmlOutput
from .MultipleChoiceInput import MultipleChoiceInput
from .CardsInput import CardsInput
from .FileOutput import FileOutput
from .RichTextInput import RichTextInput
from .VideoInput import VideoInput
from .CustomInput import CustomInput



from abc import ABC

from typing import TYPE_CHECKING

PILImage = ABC
PandasDataFrame = ABC
PlotlyFigure = ABC

if TYPE_CHECKING:
    from plotly.graph_objects import Figure
    from PIL.Image import Image
    from pandas import DataFrame

    PILImage = Image
    PandasDataFrame = DataFrame
    PlotlyFigure = Figure



import flask

from ...usage import usage
from ..utils import is_it_true
from .main import MainController
from ...execution.execution import RequestData
from ...execution.hook_execution import HookExecution
from ...repositories.project.project import ProjectRepository


def get_editor_bp(controller: MainController):
    bp = flask.Blueprint("editor_hooks", __name__)

    @bp.route("/<path:id>", methods=["GET"])
    @usage
    def _get_hook(id: str):
        hook = controller.get_hook(id)
        if not hook:
            flask.abort(404)
        return hook.editor_dto

    @bp.route("/", methods=["GET"])
    @usage
    def _get_hooks():
        return [f.editor_dto for f in controller.get_hooks()]

    @bp.route("/", methods=["POST"])
    @usage
    def _create_hook():
        hook = controller.create_hook()
        return hook.editor_dto

    @bp.route("/<path:id>", methods=["PUT"])
    @usage
    def _update_hook(id: str):
        changes = flask.request.json
        if not changes:
            flask.abort(400)

        hook = controller.update_stage(id, changes)
        return hook.editor_dto if hook else None

    @bp.route("/<path:id>", methods=["DELETE"])
    @usage
    def _delete_hook(id: str):
        remove_file = flask.request.args.get(
            "remove_file", default=False, type=is_it_true
        )
        controller.delete_hook(id, remove_file)
        return {"success": True}

    @bp.route("/<path:id>/test", methods=["POST", "GET", "PUT", "DELETE", "PATCH"])
    @usage
    def _test_hook(id: str):
        hook = controller.get_hook(id)
        if not hook:
            flask.abort(404)

        request_data = RequestData(
            method=flask.request.method,
            body=flask.request.get_data(as_text=True),
            headers=flask.request.headers,
            query_params=flask.request.args,
        )

        project = ProjectRepository.load()
        is_initial = project.is_initial(hook)
        execution = HookExecution(hook, is_initial, request_data)

        execution.run_sync()

        body, status, headers = execution.get_response()

        return {
            "body": body,
            "status": status,
            "headers": headers,
            "stdout": "".join(execution.stdout if execution else []),
            "stderr": "".join(execution.stderr if execution else []),
        }

    return bp



from . import forms as forms_contract  # exported
from .forms import Message, StdioMessage  # exported


def should_send(msg: Message, is_preview: bool) -> bool:
    return is_preview or not isinstance(msg, StdioMessage)
